# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License <http://www.gnu.org/licenses/gpl-3.0.txt>
# for more details.
#

"""Read JSON files downloaded from DevDocs.io. 

The JSON file is in format {keyword: html}. 

If keyword has a slash "element/a", the second part "a" should redirect to 
the main article "element/a". 

Each dictionary has a special entry on keyword "index". It has the name of 
the volume in h1 tag in html. 

## Usage

These alpha quality instructions are written from memory. 

Download latest aardtools and test that they work. I wrote this on Ubuntu 14.04 LTS amd64. 

$ sudo apt-get -y install build-essential python-dev python-virtualenv libicu-dev libevent-dev libxml2-dev libxslt1-dev git texlive-latex-base blahtexml mediawiki-math dvipng python-pyicu

$ virtualenv --system-site-packages env-aard
$ aardcompile --work-dir out/ jsonvolume
$ pip install -e git+git://github.com/aarddict/tools.git#egg=aardtools

Add this to compile.py:

	    from aardtools.jsonvolume import JsonVolumeArticleSource	# <-- add this
	--
	     for cls in (MediawikiArticleSource,
		# ...
		        JsonVolumeArticleSource,	# <-- add this

Run aardcompile:

$ aardcompile jsonvolume foo.json

"""
__copyright__ = "Copyright 2016 Tero Karvinen http://TeroKarvinen.com. GPL version 3. "
__version__ = "0.0.1 alpha"

from aarddict import dictionary
import collections
from aardtools.compiler import ArticleSource, Article
import json
from bs4 import BeautifulSoup as BS # sudo apt-get -y install python3-bs4

class JsonVolumeArticleSource(ArticleSource, collections.Sized):

	@classmethod
	def name(cls):
		return "jsonvolume"

	@classmethod
	def register_args(cls, parser):
		parser.add_argument("--append-html",
			help="Append this HTML to each entry",
			default="<small>Devdocs.io scraped original source, TeroKarvinen.com compiled this dictionary with aardtools.</small>",
		)
		parser.add_argument("--license-text",	# "--license" does not work here
			help="License text, added to dictionary volume info",
			default="See keyword 'index' of this dictionary volume.",
		)

	def __init__(self, args):
		super(JsonVolumeArticleSource, self).__init__(self)
		assert len(args.input_files)==1
		self.infile = args.input_files[0]
		self.appendHtml= args.append_html
		print("args.license_text %s" % args.license_text)
		self.license = args.license_text
		self.readJSONFile(self.infile)
		self.updateTitle()

	def updateTitle(self):
		html=self.words["index"]
		soup=BS(html, "html.parser")
		
		# title
		title="no_title_found"
		try:
			title=soup.h1.string
		except AttributeError:
			errorMsg=['"%s": html h1 extraction failed. ' % self.infile]
			errorMsg.append('HTML starts: "%s"' % (html[:100]))
			print(errorMsg)
		title=title.strip()
		self.title=title
		
		# description
		NO_DESC="No description found. See keyword 'index' of this dictionary volume. "
		desc=NO_DESC
		try:
			desc=soup.p.string
		except AttributeError:
			errorMsg=['"%s": html description extraction failed. ' % self.infile]
			errorMsg.append('HTML starts: "%s"' % (html[:100]))
			print(errorMsg)
		if not desc: desc=NO_DESC
		desc=desc.strip()
		self.desc=desc
					
	def readJSONFile(self, infile):
		with open(infile) as f:
			try:
				self.words=json.load(f)
				assert len(self.words)>1
			except ValueError as e:
				print('"%s": json load failed' % infile)

	def processInFile(self, infile):
		self.readJSONFile()
		self.updateTitle()

	@property
	def metadata(self):	
		meta = {
			"title": self.title,
			"description": self.desc,
			"license": self.license,
			# "version": "version_placeholder"
			# "index_language" =  "en",
			# "article_language" =  "en"
		}
		return meta

	def __len__(self):
		return 0	# todo? Article count seems to work without this

	def redirect(self, keyword, target):
		"Return Article() object that redirects keyword to target. e.g. 'a'->'element/a'"
		meta = {u'r': target}
		# some examples use tojson = functools.partial(json.dumps, ensure_ascii=False)
		serialized = json.dumps(('', [], meta))
		return Article(keyword, serialized, isredirect=True)

	def article(self, keyword, text):
		return Article(keyword, json.dumps((text, [])))

	def __iter__(self):
		for keyword, html in self.words.items():
			if self.appendHtml:
				html+=self.appendHtml
			if "/" in keyword:
				# "form" becomes the main article, "element/form" a redirect
				keyword2=keyword.split("/")[1]
				yield self.article(keyword2, html)
				yield self.redirect(keyword, keyword2)	
				# TODO: more extra keywords needed, 
				# e.g. python stdtypes should have "list" as a keyword
			else:
				yield self.article(keyword, html)

